<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2018 DevFuse
 */

namespace IPS\videos\modules\front\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Submit Controller
 */
class _submit extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Output::i()->sidebar['enabled'] = FALSE;

		parent::execute();        
        
		/* You must purchase copyright removal before removing */
        if( !\IPS\Settings::i()->devfuse_copy_num && !\IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->output .= "<div style='clear:both;text-align:center;position:absolute;bottom:15px;width:95%;'><a href='http://www.devfuse.com/' class='ipsType_light ipsType_smaller'>IP.Board Videos by DevFuse</a></div>";    
        }        
	}

	/**
	 * Choose category
	 */
	protected function manage()
	{
	    /* Can add to any category? */
        if( !\IPS\videos\Category::canOnAny('add') )
        {
            \IPS\Output::i()->error( 'video_no_category_add_perms', '', 403, '' );     
        }
       
        /* Select cat form */
        $form = new \IPS\Helpers\Form( 'form', NULL, NULL, array( 'data-bypassValidation' => true ) );
		$form->class = 'ipsForm_vertical ';

        $form->actionButtons = array();
        $form->addButton( 'videos_continue', 'submit', NULL, 'ipsButton ipsButton_primary' );

        /* Add quick add button */
        if( \IPS\Settings::i()->vs_quick_add )
        {
            $form->addButton( 'videos_quick_add', 'submit', NULL, 'ipsButton ipsButton_secondary', array( 'name' => 'quickAddButton', 'value' => 1 ) );
        }

		$form->add( new \IPS\Helpers\Form\Node( 'video_category', isset( $data['category'] ) ? $data['category'] : NULL, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit', 'front', 'videos_add' ),
			'class'					=> 'IPS\videos\Category',
            'permissionCheck'		=> function( $node )
			{ 
			    /* Cat only? */
                if( $node->cat_only )
                {
                    return FALSE;
                }			 
             
                /* Can add? */
				if ( $node->can( 'add' ) )
				{
                    return TRUE;
				}
				
				return NULL;
			}
		) ) );
        
        /* Setup quick add support */
        if( \IPS\Settings::i()->vs_quick_add )
        {
            $form->add( new \IPS\Helpers\Form\Url( 'videos_media', NULL, FALSE, array( 'placeholder' => 'https://www.youtube.com/watch?v=Oh4i5ECWblo' ), function( $val )
            {
                /* Check for duplicate media urls? */
                if( \IPS\Settings::i()->vs_duplicate_media_url )
                {
                    /* Throw error if any found */
                    $existingCount = \IPS\Db::i()->select( 'video_data', 'videos_videos', array( array( 'video_data=?', $val ) ) )->count();

                    if( $existingCount )
                    {
                        throw new \DomainException( 'duplicate_media_url' );
                    }
                }
            }, NULL, NULL, 'videos_media' ) );
        }

		if ( $values = $form->values() )
		{
		    /* Setup next step url */
            $url = \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit&do=submit', 'front', 'videos_add' )->setQueryString( array( 'category' => $values['video_category']->_id ) );

            /* Is quick add enabled? */
            if ( \IPS\Settings::i()->vs_quick_add AND $values['videos_media'] )
            {
                /* Add video right now? */
                if( isset( \IPS\Request::i()->quickAddButton ) and \IPS\Request::i()->quickAddButton )
                {
                    /* Try and get media data */
                    try
                    {
                        $mediaData = \IPS\videos\MediaSite::loadDataFromUrl( $values['videos_media'] );
                    }
                    catch( \OutOfRangeException $e ){}

                    /* We at least need a confirmed title before adding */
                    if( isset( $mediaData['title'] ) AND $mediaData['title'] )
                    {
                        $video = \IPS\videos\Video::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::ts( time() ), $values['video_category'] );
                        $video->title       = $mediaData['title'];
                        $video->description = \IPS\Text\Parser::parseStatic( $mediaData['description'], TRUE, NULL, \IPS\Member::loggedIn(), 'videos_Videos', TRUE, !(bool) \IPS\Member::loggedIn()->group['g_dohtml'] );
                        $video->video_data  = $values['videos_media'];
                        $video->video_type  = 'media_url';
                        $video->save();
                        $video->generateThumbnail();
                        $video->generateEmbed();

                        /* Any tags to save? */
                        if( $values['video_category']->tags_enable AND isset( $mediaData['tags'] ) AND count( $mediaData['tags'] ) )
                        {
                            $tags = $mediaData['tags'];

                            /* Limit tags? */
                            if( \IPS\Settings::i()->tags_max )
                            {
                                $tags = array_slice( $tags, 0, \IPS\Settings::i()->tags_max );
                            }

                            /* Add tags to video */
                            $video->setTags( $tags );
                        }

                        /* Send them to new video */
                        \IPS\Output::i()->redirect( $video->url() );
                    }
                }

                /* Pass url to add video form */
                if( $values['videos_media'] )
                {
                    $url = $url->setQueryString( array( 'media' => (string) $values['videos_media'] ) );
                }
            }

            \IPS\Output::i()->redirect( $url );
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->categorySelect( $form );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' ) );
	}

	/**
	 * Submit videos
	 *
	 * @return	void
	 */
	protected function submit()
	{	
        /* Get category */
		try
		{
			$category = \IPS\videos\Category::loadAndCheckPerms( \IPS\Request::i()->category );
            
            /* Is cat only? */
            if( $category->cat_only )
            {
                \IPS\Output::i()->error( 'cat_only_error', '', 403, '' );   
            }
            
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit', 'front', 'videos_add' ) );
		}

        /* Club or normal story? */
        if ( \IPS\IPS::classUsesTrait( $category, 'IPS\Content\ClubContainer' ) and $club = $category->club() )
        {
            \IPS\core\FrontNavigation::$clubTabActive = TRUE;
            \IPS\Output::i()->breadcrumb = array();
            \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
            \IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );

            if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
            {
                \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $category, 'sidebar' );
            }
        }

        /* Show form */
		\IPS\Output::i()->title = $category->_title;
		\IPS\Output::i()->breadcrumb[] = array( NULL, $category->_title );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('submit')->formPage( $category, \IPS\videos\Video::create( $category ) ); 	   
	}
}